//MyKeypad_I2C.cpp
//Code fuer Arduino und Attiny
//Author Retian
//Version 2.1


/*
Ansteuerung einer bis zu 16-stelligen Tastatur (4x4 Tasten) mit dem PCF8574 Portexpander.
Es koennen auch 4-, 9- oder 12-stellige Matrix-Tastaturen (1x4, 3x3 oder 3x4) angesteuert werden.

Zur fluessigen Tastaturabfrage sollte die Funktion vom Hauptprogramm ca. alle 100-200 ms (typisch 150 ms)abgefragt werden.

MyKeypad_I2C Name(I2C-Adresse);

Beispiel siehe unter:
http://arduino-projekte.webnode.at/meine-libraries/16er-tastatur-mit-i2c/

Funktionen siehe unter:
http://arduino-projekte.webnode.at/meine-libraries/16er-tastatur-mit-i2c/funktionen/

*/

//*************************************************************************
//*************************************************************************

#include "Arduino.h"
#include "MyKeypad_I2C.h"

MyKeypad_I2C::MyKeypad_I2C(byte i2cAdd)
{
  _i2cAdd = i2cAdd;
  Wire.begin();

  outByte[0] = 0xFE;
  outByte[1] = 0xFD;
  outByte[2] = 0xFB;
  outByte[3] = 0xF7;

  //Vorbelegte Tastencodes fuer verschiedene Tastaturen 
  //Je nach Anschluss der Tastaturmatrix am PCF8574 (Zeilen- und Spaltenanschluesse)
  //aendern sich die Tastencodes und muessen dann neu vorgegeben werden.

  //Die Tastencodes von nicht verwendeten Tastaturen mssen als Kommentare markiert werden!
  
  /*
  // Tastenbelegung      Rueckgabewerte
  //  7  8  9  A          7  8  9 10
  //  4  5  6  B          4  5  6 11
  //  1  2  3  C          1  2  3 12
  //  0  F  E  D          0 15 14 13
  
  keyCode[0]  = 0x77; //Tastencode fuer Rueckgabewert 0
  keyCode[1]  = 0xB7; //Tastencode fuer Rueckgabewert 1
  keyCode[2]  = 0xBB; //Tastencode fuer Rueckgabewert 2
  keyCode[3]  = 0xBD; //Tastencode fuer Rueckgabewert 3
  keyCode[4]  = 0xE7; //Tastencode fuer Rueckgabewert 4
  keyCode[5]  = 0xEB; //Tastencode fuer Rueckgabewert 5
  keyCode[6]  = 0xED; //Tastencode fuer Rueckgabewert 6
  keyCode[7]  = 0xD7; //Tastencode fuer Rueckgabewert 7
  keyCode[8]  = 0xDB; //Tastencode fuer Rueckgabewert 8
  keyCode[9]  = 0xDD; //Tastencode fuer Rueckgabewert 9
  keyCode[10] = 0xDE; //Tastencode fuer Rueckgabewert 10
  keyCode[11] = 0xEE; //Tastencode fuer Rueckgabewert 11
  keyCode[12] = 0xBE; //Tastencode fuer Rueckgabewert 12
  keyCode[13] = 0x7E; //Tastencode fuer Rueckgabewert 13
  keyCode[14] = 0x7D; //Tastencode fuer Rueckgabewert 14
  keyCode[15] = 0x7B; //Tastencode fuer Rueckgabewert 15
  */
  
  // Tastenbelegung       Rueckgabewert
  //  1  2  3              1  2  3
  //  4  5  6              4  5  6
  //  7  8  9              7  8  9
  //  *  0  #             11  0 13 
  
  keyCode[0]  = 0xDE; //Tastencode fuer Rueckgabewert 0
  keyCode[1]  = 0xE7; //Tastencode fuer Rueckgabewert 1
  keyCode[2]  = 0xD7; //Tastencode fuer Rueckgabewert 2
  keyCode[3]  = 0xB7; //Tastencode fuer Rueckgabewert 3
  keyCode[4]  = 0xEB; //Tastencode fuer Rueckgabewert 4
  keyCode[5]  = 0xDB; //Tastencode fuer Rueckgabewert 5
  keyCode[6]  = 0xBB; //Tastencode fuer Rueckgabewert 6
  keyCode[7]  = 0xED; //Tastencode fuer Rueckgabewert 7
  keyCode[8]  = 0xDD; //Tastencode fuer Rueckgabewert 8
  keyCode[9]  = 0xBD; //Tastencode fuer Rueckgabewert 9
  keyCode[11] = 0xEE; //Tastencode fuer Rueckgabewert 11
  keyCode[13] = 0xBE; //Tastencode fuer Rueckgabewert 13
  
  /*
  // Tastenbelegung      Rueckgabewerte
  //  1  2  3  A          1  2  3 10
  //  4  5  6  B          4  5  6 11
  //  7  8  9  C          7  8  9 12
  //  *  0  #  D         15  0 14 13
  
  keyCode[0]  = 0xDE; //Tastencode fuer Rueckgabewert 0
  keyCode[1]  = 0xE7; //Tastencode fuer Rueckgabewert 1
  keyCode[2]  = 0xD7; //Tastencode fuer Rueckgabewert 2
  keyCode[3]  = 0xB7; //Tastencode fuer Rueckgabewert 3
  keyCode[4]  = 0xEB; //Tastencode fuer Rueckgabewert 4
  keyCode[5]  = 0xDB; //Tastencode fuer Rueckgabewert 5
  keyCode[6]  = 0xBB; //Tastencode fuer Rueckgabewert 6
  keyCode[7]  = 0xED; //Tastencode fuer Rueckgabewert 7
  keyCode[8]  = 0xDD; //Tastencode fuer Rueckgabewert 8
  keyCode[9]  = 0xBD; //Tastencode fuer Rueckgabewert 9
  keyCode[10] = 0x77; //Tastencode fuer Rueckgabewert 10
  keyCode[11] = 0x7B; //Tastencode fuer Rueckgabewert 11
  keyCode[12] = 0x7D; //Tastencode fuer Rueckgabewert 12
  keyCode[13] = 0x7E; //Tastencode fuer Rueckgabewert 13
  keyCode[14] = 0xBE; //Tastencode fuer Rueckgabewert 14
  keyCode[15] = 0xEE; //Tastencode fuer Rueckgabewert 15
  
  
  // Tastenbelegung      Rueckgabewerte
  //  1  2  3  4          1  2  3  4
  
  keyCode[1]  = 0xED; //Tastencode fuer Rueckgabewert 1
  keyCode[2]  = 0xEE; //Tastencode fuer Rueckgabewert 2
  keyCode[3]  = 0xE7; //Tastencode fuer Rueckgabewert 3
  keyCode[4]  = 0xEB; //Tastencode fuer Rueckgabewert 4
  */
}


//*************************************************************************
//Tastatur (PCF8574-Portexpander) vorhanden ?

bool MyKeypad_I2C::isReady()
{
  Wire.beginTransmission(_i2cAdd);
  if (Wire.endTransmission() == 0) return true;
  else return false;
}


//*************************************************************************
//Setzen eines Tastencode

void MyKeypad_I2C::setKeyCode(byte code, byte index)
{
  byte _index = index;
  byte _code = code;
  
  keyCode[_index] = _code;
}


//*************************************************************************
//Loeschen aller Tastencodes

void MyKeypad_I2C::clearAllKeyCodes()
{
  for (byte i = 0; i < 16; i++) keyCode[i] = 0xFF;
}


//*************************************************************************
//Abfrage des Tastaturwertes

byte MyKeypad_I2C::receiveKey(int8_t anzahl)
{
  int8_t _anzahl = anzahl;
  byte keyVal = 255;
  byte inByte;
  byte inCode;
  byte add;
  static byte abfrageZaehler = 0;

  //Pause vor erster Tastenwiederholung (in Anzahl von Abfragezyklen)
  byte anzZyklusPause = 2;

  if (_anzahl > 0) _anzahl += anzZyklusPause;

  for (byte i = 0; i < 4; i++)
  {
    Wire.beginTransmission(_i2cAdd);
    Wire.write(outByte[i]);
    Wire.endTransmission();
        
    Wire.requestFrom(int(_i2cAdd), 1);
    while(Wire.available() == 0);
    inByte = ((Wire.read() & 0xF0) >> 4);
    if (inByte < 0x0F)
    {
      inCode = (outByte[i] << 4) | inByte;
      for (byte j = 0; j < 16; j++)
      {
        if (inCode == keyCode[j])keyVal = j;
      }
    }   
  }

  if (abfrageZaehler > 0 && abfrageZaehler <= anzZyklusPause)
  {
    abfrageZaehler++;
    return(255);
  }
   
  if (_anzahl > 0)
  {
    if (keyVal == 255)
    {
      abfrageZaehler = 0;
      add = 0;
    }
    if (keyVal != 255) abfrageZaehler++;
    if (abfrageZaehler > _anzahl) add = 100;
    return (keyVal + add);
  }
  else if (_anzahl == 0)
  {
    if (keyVal == 255) abfrageZaehler = 0;
    if (keyVal != 255) abfrageZaehler++;
    return (keyVal);
  }
  else if (_anzahl == -1)
  {
    if (keyVal == 255) abfrageZaehler = 0;
    if (keyVal != 255) abfrageZaehler++;
    if (abfrageZaehler > 1) keyVal = 255;
    return (keyVal);
  }
}


//*************************************************************************
//Ausgabe des Tastencode der gedrueckten Taste (Nicht fr Attiny)
//Nur zur Anpassung der Tastatur

void MyKeypad_I2C::checkKey()
{
#if defined_ATtiny
  //Keine Aktivitaet, wenn ein Attiny eingesetzt wird
#else
  byte inByte;
  byte inCode;
  byte keyVal;
    
  for (byte i = 0; i < 4; i++)
  {
    Wire.beginTransmission(_i2cAdd);
    Wire.write(outByte[i] | 0xF0);
    Wire.endTransmission();
    
    Wire.requestFrom(int(_i2cAdd), 1);
    while(Wire.available() == 0);
    inByte = ((Wire.read() & 0xF0) >> 4);
    
    if (inByte < 0x0F)
    {
      inCode = (outByte[i] << 4) | inByte;
      Serial.print("Tastencode = ");
      Serial.print("0x");
      Serial.print(inCode, HEX);
	 
      for (byte j = 0; j < 16; j++)
      {
        if (inCode == keyCode[j]) keyVal = j;
      }
      
      Serial.print(" -> Rueckgabewert = ");
      Serial.println(keyVal);
    }
  }
#endif
}

//*************************************************************************
//Ausgabe aller Tastencodes am Seriellen Monitor (Nicht fr Attiny)
//Nur zur Anpassung der Tastatur

void MyKeypad_I2C::printKeyCodes()
{
#if defined_ATtiny
  //Keine Aktivitaet, wenn ein Attiny eingesetzt wird
#else
  Serial.println();
  Serial.println("*********** Tastencodes ***********");
  for (byte i = 0; i < 16; i++)
  {
    if (keyCode[i] != 0)
    {
	  Serial.print("Tastencode = 0x");
	  Serial.print(keyCode[i], HEX);
      Serial.print(" -> Rueckgabewert = ");
	  if (i < 10) Serial.print(" ");
	  Serial.println(i);
	}
  }
  Serial.println();
#endif
}
